@@ -6,6 +6,7 @@ import decimal
 import enum
 import inspect
 import json
+import logging
 import random
 import sys
 import typing
@@ -18,15 +19,18 @@ from pytz import utc
 
 from dataclasses_avroschema import schema_generator, serialization, types, utils
 
+from . import field_utils
 from .exceptions import InvalidMap, NameSpaceRequiredException
+from .types import JsonDict
 
 PY_VER = sys.version_info
 
-if PY_VER >= (3, 9):
+if PY_VER >= (3, 9):  # pragma: no cover
     GenericAlias = (typing.GenericAlias, typing._GenericAlias, typing._SpecialGenericAlias, typing._UnionGenericAlias)  # type: ignore # noqa: E501
 else:
-    GenericAlias = typing._GenericAlias  # type: ignore
+    GenericAlias = typing._GenericAlias  # type: ignore  # pragma: no cover
 
+logger = logging.getLogger(__name__)
 
 fake = Faker()
 p = inflect.engine()
@@ -44,13 +48,13 @@ ENUM = "enum"
 MAP = "map"
 FIXED = "fixed"
 DATE = "date"
-TIME_MILLIS = "time-millis"
-TIMESTAMP_MILLIS = "timestamp-millis"
 UUID = "uuid"
 DECIMAL = "decimal"
 LOGICAL_DATE = {"type": INT, "logicalType": DATE}
-LOGICAL_TIME = {"type": INT, "logicalType": TIME_MILLIS}
-LOGICAL_DATETIME = {"type": LONG, "logicalType": TIMESTAMP_MILLIS}
+LOGICAL_TIME_MILIS = {"type": INT, "logicalType": field_utils.TIME_MILLIS}
+LOGICAL_TIME_MICROS = {"type": LONG, "logicalType": field_utils.TIME_MICROS}
+LOGICAL_DATETIME_MILIS = {"type": LONG, "logicalType": field_utils.TIMESTAMP_MILLIS}
+LOGICAL_DATETIME_MICROS = {"type": LONG, "logicalType": field_utils.TIMESTAMP_MICROS}
 LOGICAL_UUID = {"type": STRING, "logicalType": UUID}
 
 PYTHON_TYPE_TO_AVRO = {
@@ -68,8 +72,8 @@ PYTHON_TYPE_TO_AVRO = {
     types.Int32: INT,
     types.Float32: FLOAT,
     datetime.date: {"type": INT, "logicalType": DATE},
-    datetime.time: {"type": INT, "logicalType": TIME_MILLIS},
-    datetime.datetime: {"type": LONG, "logicalType": TIMESTAMP_MILLIS},
+    datetime.time: {"type": INT, "logicalType": field_utils.TIME_MILLIS},
+    datetime.datetime: {"type": LONG, "logicalType": field_utils.TIMESTAMP_MILLIS},
     uuid.uuid4: {"type": STRING, "logicalType": UUID},
 }
 
@@ -78,7 +82,16 @@ PYTHON_INMUTABLE_TYPES = (str, int, types.Int32, types.Float32, bool, float, byt
 
 PYTHON_PRIMITIVE_CONTAINERS = (list, tuple, dict)
 
-PYTHON_LOGICAL_TYPES = (datetime.date, datetime.time, datetime.datetime, uuid.uuid4, uuid.UUID, decimal.Decimal)
+PYTHON_LOGICAL_TYPES = (
+    datetime.date,
+    datetime.time,
+    types.TimeMicro,
+    datetime.datetime,
+    types.DateTimeMicro,
+    uuid.uuid4,
+    uuid.UUID,
+    decimal.Decimal,
+)
 
 PYTHON_PRIMITIVE_TYPES = PYTHON_INMUTABLE_TYPES + PYTHON_PRIMITIVE_CONTAINERS
 
@@ -104,18 +117,20 @@ PythonImmutableTypes = typing.Union[
 
 @dataclasses.dataclass  # type: ignore
 class BaseField:
-    avro_type: typing.ClassVar[str]
-
     name: str
     type: typing.Any  # store the python primitive type
     default: typing.Any
     parent: typing.Any
-    metadata: typing.Mapping = dataclasses.field(default_factory=dict)
+    metadata: typing.Optional[typing.Mapping] = None
     model_metadata: typing.Optional[utils.SchemaMetadata] = None
 
     def __post_init__(self) -> None:
         self.model_metadata = self.model_metadata or utils.SchemaMetadata()
 
+    @property
+    def avro_type(self) -> typing.Union[str, typing.Dict]:
+        ...  # pragma: no cover
+
     @staticmethod
     def _get_self_reference_type(a_type: typing.Any) -> str:
         internal_type = a_type.__args__[0]
@@ -126,18 +141,21 @@ class BaseField:
     def get_singular_name(name: str) -> str:
         singular = p.singular_noun(name)
 
-        if singular:
+        # do the check because of mypy.
+        # p.singular_noun returns Union[str, bool]
+        if isinstance(singular, str):
             return singular
         return name
 
     def get_metadata(self) -> typing.List[typing.Tuple[str, str]]:
         meta_data_for_template = []
-        try:
-            metadata = dict(self.metadata)
-            for name, value in metadata.items():
-                meta_data_for_template.append((name, value))
-        except (ValueError, TypeError):
-            return meta_data_for_template
+
+        if self.metadata is not None:
+            try:
+                for name, value in self.metadata.items():
+                    meta_data_for_template.append((name, value))
+            except (ValueError, TypeError):  # pragma: no cover
+                logger.warn("Error during getting metadata")  # pragma: no cover
         return meta_data_for_template
 
     def render(self) -> OrderedDict:
@@ -274,13 +292,19 @@ class BytesField(ImmutableField):
 
 @dataclasses.dataclass
 class NoneField(ImmutableField):
-    avro_type: typing.ClassVar[str] = NULL
+    @property
+    def avro_type(self) -> typing.Union[str, typing.Dict]:
+        return NULL
 
 
 @dataclasses.dataclass
 class ContainerField(BaseField):
     default_factory: typing.Optional[typing.Callable] = None
 
+    @property
+    def avro_type(self) -> typing.Dict:
+        ...  # pragma: no cover
+
     def get_avro_type(self) -> PythonImmutableTypes:
         avro_type = self.avro_type
         avro_type["name"] = self.get_singular_name(self.name)
@@ -354,11 +378,11 @@ class DictField(ContainerField):
             raise InvalidMap(self.name, key_type)
 
     @property
-    def avro_type(self) -> typing.Dict[str, typing.Any]:
+    def avro_type(self) -> JsonDict:
         self.generate_values_type()
         return {"type": MAP, "values": self.values_type}
 
-    def get_default_value(self) -> typing.Union[typing.Dict[str, typing.Any], dataclasses._MISSING_TYPE]:
+    def get_default_value(self) -> typing.Union[JsonDict, dataclasses._MISSING_TYPE]:
         if self.default is None:
             return {}
         elif callable(self.default_factory):
@@ -467,7 +491,7 @@ class UnionField(BaseField):
 
 @dataclasses.dataclass
 class FixedField(BaseField):
-    def get_avro_type(self) -> typing.Dict[str, typing.Any]:
+    def get_avro_type(self) -> JsonDict:
         avro_type = {
             "type": FIXED,
             "name": self.get_singular_name(self.name),
@@ -504,7 +528,7 @@ class EnumField(BaseField):
     def get_symbols(self) -> typing.List[str]:
         return [member.value for member in self.type if member.name != "Meta"]
 
-    def get_avro_type(self) -> typing.Dict[str, typing.Any]:
+    def get_avro_type(self) -> JsonDict:
         avro_type = {
             "type": ENUM,
             "name": self.get_singular_name(self.name),
@@ -572,7 +596,9 @@ class DateField(LogicalTypeField):
     the number of days from the unix epoch, 1 January 1970 (ISO calendar).
     """
 
-    avro_type: typing.ClassVar[str] = LOGICAL_DATE
+    @property
+    def avro_type(self) -> typing.Dict:
+        return LOGICAL_DATE
 
     @staticmethod
     def to_avro(date: datetime.date) -> int:
@@ -597,7 +623,7 @@ class DateField(LogicalTypeField):
 
 
 @dataclasses.dataclass
-class TimeField(LogicalTypeField):
+class TimeMilliField(LogicalTypeField):
     """
     The time-millis logical type represents a time of day,
     with no reference to a particular calendar,
@@ -607,7 +633,9 @@ class TimeField(LogicalTypeField):
     where the int stores the number of milliseconds after midnight, 00:00:00.000.
     """
 
-    avro_type: typing.ClassVar[str] = LOGICAL_TIME
+    @property
+    def avro_type(self) -> typing.Dict:
+        return LOGICAL_TIME_MILIS
 
     @staticmethod
     def to_avro(time: datetime.time) -> int:
@@ -634,6 +662,48 @@ class TimeField(LogicalTypeField):
         return fake.time_object()
 
 
+@dataclasses.dataclass
+class TimeMicroField(LogicalTypeField):
+    """
+    The time-micros logical type represents a time of day,
+    with no reference to a particular calendar,
+    time zone or date, with a precision of one millisecond.
+
+    A time-micros logical type annotates an Avro long,
+    where the int stores the number of milliseconds after midnight, 00:00:00.000000.
+    """
+
+    @property
+    def avro_type(self) -> typing.Dict:
+        return LOGICAL_TIME_MICROS
+
+    @staticmethod
+    def to_avro(time: datetime.time) -> float:
+        """
+        Returns the number of microseconds after midnight, 00:00:00.000000
+        for a given time object
+
+        Arguments:
+            time (datetime.time)
+
+        Returns:
+            float
+        """
+        hour, minutes, seconds, microseconds = (
+            time.hour,
+            time.minute,
+            time.second,
+            time.microsecond,
+        )
+
+        return float((((hour * 60 + minutes) * 60 + seconds) * 1000000) + microseconds)
+
+    def fake(self) -> datetime.time:
+        datetime_object: datetime.datetime = fake.date_time(tzinfo=utc)
+        datetime_object = datetime_object + datetime.timedelta(microseconds=random.randint(0, 999))
+        return datetime_object.time()
+
+
 @dataclasses.dataclass
 class DatetimeField(LogicalTypeField):
     """
@@ -645,7 +715,9 @@ class DatetimeField(LogicalTypeField):
     1 January 1970 00:00:00.000 UTC.
     """
 
-    avro_type: typing.ClassVar[str] = LOGICAL_DATETIME
+    @property
+    def avro_type(self) -> typing.Dict:
+        return LOGICAL_DATETIME_MILIS
 
     @staticmethod
     def to_avro(date_time: datetime.datetime) -> float:
@@ -670,9 +742,50 @@ class DatetimeField(LogicalTypeField):
         return fake.date_time(tzinfo=utc)
 
 
+@dataclasses.dataclass
+class DatetimeMicroField(LogicalTypeField):
+    """
+    The timestamp-millis logical type represents an instant on the global timeline,
+    independent of a particular time zone or calendar, with a precision of one millisecond.
+
+    A timestamp-millis logical type annotates an Avro long,
+    where the long stores the number of milliseconds from the unix epoch,
+    1 January 1970 00:00:00.000000 UTC.
+    """
+
+    @property
+    def avro_type(self) -> typing.Dict:
+        return LOGICAL_DATETIME_MICROS
+
+    @staticmethod
+    def to_avro(date_time: datetime.datetime) -> float:
+        """
+        Returns the number of milliseconds from the unix epoch,
+        1 January 1970 00:00:00.000000 UTC for a given datetime
+
+        Arguments:
+            date_time (datetime.datetime)
+
+        Returns:
+            float
+        """
+        if date_time.tzinfo:
+            ts = (date_time - utils.epoch).total_seconds()
+        else:
+            ts = (date_time - utils.epoch_naive).total_seconds()
+
+        return ts * 1000000
+
+    def fake(self) -> datetime.datetime:
+        datetime_object: datetime.datetime = fake.date_time(tzinfo=utc)
+        return datetime_object + datetime.timedelta(microseconds=random.randint(0, 999))
+
+
 @dataclasses.dataclass
 class UUIDField(LogicalTypeField):
-    avro_type: typing.ClassVar[str] = LOGICAL_UUID
+    @property
+    def avro_type(self) -> typing.Dict:
+        return LOGICAL_UUID
 
     def validate_default(self) -> bool:
         msg = f"Invalid default type. Default should be {str} or {uuid.UUID}"
@@ -691,7 +804,7 @@ class UUIDField(LogicalTypeField):
 @dataclasses.dataclass
 class RecordField(BaseField):
     def get_avro_type(self) -> typing.Union[typing.List, typing.Dict]:
-        meta = getattr(self.type, "Meta", None)
+        meta = getattr(self.type, "Meta", type)
         metadata = utils.SchemaMetadata.create(meta)
 
         alias = self.parent.metadata.get_alias_nested_items(self.name) or metadata.get_alias_nested_items(self.name)  # type: ignore  # noqa E501
@@ -704,9 +817,9 @@ class RecordField(BaseField):
 
         if not self.exist_type() or alias is not None:
             user_defined_type = utils.UserDefinedType(name=name, type=self.type)
-            self.parent.user_defined_types += (user_defined_type,)
+            self.parent.user_defined_types.add(user_defined_type)
 
-            record_type = self.type.avro_schema_to_python(root=self.parent)
+            record_type = self.type.avro_schema_to_python(parent=self.parent)
             record_type["name"] = name
         else:
             if metadata.namespace is None:
@@ -780,7 +893,7 @@ class DecimalField(BaseField):
             #
             # self.precision = decimal.Context().prec
 
-    def get_avro_type(self) -> typing.Dict[str, typing.Any]:
+    def get_avro_type(self) -> typing.Union[JsonDict, typing.List[typing.Union[str, JsonDict]]]:
         avro_type = {"type": BYTES, "logicalType": DECIMAL, "precision": self.precision, "scale": self.scale}
         if not isinstance(self.default, decimal.Decimal) and self.default.default is None:
             return ["null", avro_type]
@@ -826,8 +939,10 @@ CONTAINER_FIELDS_CLASSES = {
 
 LOGICAL_TYPES_FIELDS_CLASSES = {
     datetime.date: DateField,
-    datetime.time: TimeField,
+    datetime.time: TimeMilliField,
+    types.TimeMicro: TimeMicroField,
     datetime.datetime: DatetimeField,
+    types.DateTimeMicro: DatetimeMicroField,
     uuid.uuid4: UUIDField,
     uuid.UUID: UUIDField,
     bytes: BytesField,
@@ -835,7 +950,7 @@ LOGICAL_TYPES_FIELDS_CLASSES = {
 }
 
 PRIMITIVE_LOGICAL_TYPES_FIELDS_CLASSES = {
-    **INMUTABLE_FIELDS_CLASSES,
+    **INMUTABLE_FIELDS_CLASSES,  # type: ignore
     **LOGICAL_TYPES_FIELDS_CLASSES,  # type: ignore
     types.Fixed: FixedField,
 }
@@ -856,7 +971,7 @@ FieldType = typing.Union[
     EnumField,
     SelfReferenceField,
     DateField,
-    TimeField,
+    TimeMilliField,
     DatetimeField,
     UUIDField,
     RecordField,
@@ -871,9 +986,12 @@ def field_factory(
     *,
     default: typing.Any = dataclasses.MISSING,
     default_factory: typing.Any = dataclasses.MISSING,
-    metadata: typing.Mapping = dataclasses.field(default_factory=dict),
-    model_metadata: typing.Optional[utils.SchemaMetadata] = None,
+    metadata: typing.Optional[typing.Mapping] = None,
+    model_metadata: utils.SchemaMetadata = None,
 ) -> FieldType:
+    if metadata is None:
+        metadata = {}
+
     if native_type in PYTHON_INMUTABLE_TYPES:
         klass = INMUTABLE_FIELDS_CLASSES[native_type]
         return klass(
@@ -902,6 +1020,17 @@ def field_factory(
             model_metadata=model_metadata,
             parent=parent,
         )
+    elif native_type in PYTHON_LOGICAL_TYPES:
+        klass = LOGICAL_TYPES_FIELDS_CLASSES[native_type]  # type: ignore
+
+        return klass(
+            name=name,
+            type=native_type,
+            default=default,
+            metadata=metadata,
+            model_metadata=model_metadata,
+            parent=parent,
+        )
     elif isinstance(native_type, GenericAlias):  # type: ignore
         origin = native_type.__origin__
 
@@ -932,16 +1061,6 @@ def field_factory(
             model_metadata=model_metadata,
             parent=parent,
         )
-    elif native_type in PYTHON_LOGICAL_TYPES:
-        klass = LOGICAL_TYPES_FIELDS_CLASSES[native_type]  # type: ignore
-        return klass(
-            name=name,
-            type=native_type,
-            default=default,
-            metadata=metadata,
-            model_metadata=model_metadata,
-            parent=parent,
-        )
     elif inspect.isclass(native_type) and issubclass(native_type, enum.Enum):
         return EnumField(
             name=name,
