@@ -1,13 +1,26 @@
 import dataclasses
+import datetime
+import sys
 import typing
 
-try:
-    # Only available in 3.9+
+if sys.version_info >= (3, 9):
     from typing import Annotated
-except ImportError:  # pragma: no cover
-    from typing_extensions import Annotated  # pragma: no cover
+else:
+    from typing_extensions import Annotated  # type: ignore # pragma: no cover
 
 T = typing.TypeVar("T")
+JsonDict = typing.Dict[str, typing.Any]
+
+CUSTOM_TYPES = (
+    "Fixed",
+    "Decimal",
+    "Int32",
+    "Float32",
+    "TimeMicro",
+    "DateTimeMicro",
+)
+
+__all__ = CUSTOM_TYPES
 
 
 class MissingSentinel(typing.Generic[T]):
@@ -58,6 +71,5 @@ class Decimal(typing.Generic[T]):
 
 Int32 = Annotated[int, "Int32"]
 Float32 = Annotated[float, "Float32"]
-
-
-CUSTOM_TYPES = ("Fixed", "Decimal", "Int32", "Float32")
+TimeMicro = Annotated[datetime.time, "TimeMicro"]
+DateTimeMicro = Annotated[datetime.datetime, "DateTimeMicro"]
