@@ -4,6 +4,7 @@ from datetime import datetime
 
 from pytz import utc
 
+from . import field_utils
 from .types import CUSTOM_TYPES
 
 try:
@@ -14,22 +15,22 @@ except ImportError:  # pragma: no cover
 try:
     import pydantic  # pragma: no cover
 except ImportError:  # type: ignore # pragma: no cover
-    pydantic = None
+    pydantic = None  # type: ignore # pragma: no cover
 
 
-def is_pydantic_model(klass: typing.Any) -> bool:
+def is_pydantic_model(klass: type) -> bool:
     if pydantic is not None:
         return issubclass(klass, pydantic.BaseModel)
     return False
 
 
-def is_faust_model(klass: typing.Any) -> bool:
+def is_faust_model(klass: type) -> bool:
     if faust is not None:
         return issubclass(klass, faust.Record)
     return False
 
 
-def is_union(a_type: typing.Any) -> bool:
+def is_union(a_type: type) -> bool:
     """
     Given a python type, return True if is typing.Union, otherwise False
 
@@ -42,7 +43,7 @@ def is_union(a_type: typing.Any) -> bool:
     return isinstance(a_type, typing._GenericAlias) and a_type.__origin__ is typing.Union  # type: ignore
 
 
-def is_self_referenced(a_type: typing.Any) -> bool:
+def is_self_referenced(a_type: type) -> bool:
     """
     Given a python type, return True if is self referenced, meaning
     that is instance of typing.ForwardRef, otherwise False
@@ -79,15 +80,19 @@ class SchemaMetadata:
     namespace: typing.Optional[typing.List[str]] = None
     aliases: typing.Optional[typing.List[str]] = None
     alias_nested_items: typing.Dict[str, str] = field(default_factory=dict)
+    time_precision: str = field_utils.TIME_MILLIS
+    datetime_precision: str = field_utils.TIMESTAMP_MILLIS
 
     @classmethod
-    def create(cls: typing.Any, klass: type) -> typing.Any:
+    def create(cls: typing.Type["SchemaMetadata"], klass: type) -> typing.Any:
         return cls(
             schema_name=getattr(klass, "schema_name", None),
             schema_doc=getattr(klass, "schema_doc", True),
             namespace=getattr(klass, "namespace", None),
             aliases=getattr(klass, "aliases", None),
             alias_nested_items=getattr(klass, "alias_nested_items", {}),
+            time_precision=getattr(klass, "time_precision", field_utils.TIME_MILLIS),
+            datetime_precision=getattr(klass, "datetime_precision", field_utils.TIMESTAMP_MILLIS),
         )
 
     def get_alias_nested_items(self, name: str) -> typing.Optional[str]:
@@ -101,7 +106,7 @@ class FieldMetadata:
     namespace: typing.Optional[str] = None
 
     @classmethod
-    def create(cls: typing.Any, klass: type) -> "FieldMetadata":
+    def create(cls: typing.Type["FieldMetadata"], klass: typing.Optional[type]) -> "FieldMetadata":
         return cls(
             aliases=getattr(klass, "aliases", []),
             doc=getattr(klass, "doc", None),
